<?php

require __DIR__ . "/vendor/autoload.php";
require __DIR__ . "/vendor/zeedhi/framework/bootstrap.php";
require __DIR__ . "/vendor/teknisa/libraries/scripts/bootstrap.php";

$instanceManager = \Zeedhi\Framework\DependencyInjection\InstanceManager::getInstance();
$routes = array(__DIR__ . "/vendor/teknisa/libraries/libRoutes.json"); // push teknisa libraries route file

$mapping = getMapping();
loadConfigFiles($instanceManager);
configAcl($instanceManager, $routes);
configPreferences($instanceManager, $routes);
loadRoutesAndServices($mapping, $instanceManager, $routes);

$instanceManager->compile();

function getMapping() {
	$mapping = file_get_contents(__DIR__ . "/config/servicesMapping.json");
	return json_decode($mapping, true);
}

function loadRoutesAndServices($mapping, $instanceManager, &$routes) {
	foreach ($mapping as $path) {
		$ext = getFilePathExtension($path);
		if($ext === 'xml')
			$instanceManager->loadFromFile(__DIR__ . $path);
		if($ext === 'json')
			$routes[] = __DIR__ . $path;
	}

	$instanceManager->registerService("routeParser", new \Zeedhi\Framework\Routing\Parsers\JSONFormat($routes));
}

function loadConfigFiles($instanceManager) {
	$db 	= __DIR__ . "/../../../backend/config/db.xml"; // prioritize product database confi
	$report = __DIR__ . "/config/report.xml"; // prioritize microservice report config
	$parameters = __DIR__ . "/../../../backend/config/parameters.xml"; // prioritize product parameters
	$general = __DIR__ . "/../../../backend/config/general.xml"; // prioritize product general services

	if(!realpath($db))
		$db = __DIR__ . "/config/db.xml";

	if(!realpath($report))
		$report = __DIR__ . "/../../../backend/config/report.xml";

	if (realpath($parameters))
		$instanceManager->loadFromFile($parameters);

	if (realpath($general))
		$instanceManager->loadFromFile($general);

	$instanceManager->loadFromFile(__DIR__ . "/config/environment.xml");
	$instanceManager->loadFromFile($db);
	$instanceManager->loadFromFile($report);
}

function configAcl($instanceManager, &$routes) {
	$container = $instanceManager->getContainer();
	$aclParam = "use_zhu_acl";

	if ($container->hasParameter($aclParam) && $container->getParameter($aclParam)) {
		$acl = __DIR__ . "/config/acl.xml";

		if(realpath($acl))
			$instanceManager->loadFromFile($acl);

		require __DIR__ . "/vendor/zeedhi/zhuaclbackend/service/bootstrap.php";
		$routes[] = __DIR__ . "/vendor/zeedhi/zhuaclbackend/config/routes.json";
	}
}

function configPreferences($instanceManager, &$routes) {
	$routes[] = __DIR__ . "/vendor/zeedhi/preferences/routes.json";
	$instanceManager->loadFromFile(__DIR__.'/vendor/zeedhi/preferences/services.xml');
}

function getFilePathExtension($path) {
	return substr(strrchr($path, '.'), 1);
}